/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "holdem.h"

void holdemGameInit(holdemgame_t *game) {
  uint8_t i;
  holdemplayer_t *player;

  // Set Blinds to nil.
  game->blindBig = 0;
  game->blindSmall = 0;

  // Reset the players
  for(i = 0; i < HOLDEM_PLAYER_COUNT; i++) {
    player = game->players + i;

    // Set their state to 0
    player->state = 0x00;

    // Set their chips to zero
    player->chips = 0;
  }
}

void holdemRoundInit(holdemgame_t *game) {
  uint8_t i;
  holdemplayer_t *player;

  // Refill the deck
  cardDeckFill(game->deck);
  game->deckSize = CARD_DECK_SIZE;

  // Reset the pot
  game->pot = 0;

  // Reset the cards
  game->cardsFacing = 0;
  
  // Reset the players
  for(i = 0; i < HOLDEM_PLAYER_COUNT; i++) {
    player = game->players + i;

    // Clear Round State(s)
    player->state &= ~(
      HOLDEM_STATE_FOLDED | 
      HOLDEM_STATE_SHOWING
    );

    player->cardCount = 0;

    // Set the players' current bet to zero.
    player->currentBet = 0;
  }
}

void holdemDeal(holdemgame_t *game, holdemplayer_t *player) {
  cardDeal(game->deck, player->cards, game->deckSize, player->cardCount);
  game->deckSize--;
  player->cardCount++;
}

void holdemDealAll(holdemgame_t *game, uint8_t count) {
  uint8_t i, j;
  for(i = 0; i < count; i++) {
    for(j = 0; j < HOLDEM_PLAYER_COUNT; j++) {
      holdemDeal(game, game->players + j);
    }
  }
}

void holdemFlop(holdemgame_t *game) {
  if(game->cardsFacing >= HOLDEM_DEALER_HAND) return;
  uint8_t i, count;


  // Burn the card off the top
  game->deckSize -= 1;

  // Change count depending on facing
  count = game->cardsFacing == 0 ? 0x03 : 0x01;

  // Deal
  for(i = 0; i < count; i++) {
    cardDeal(game->deck, game->cards, game->deckSize, game->cardsFacing);
    game->deckSize -= 1;
    game->cardsFacing += 1;
  }
}

void holdemRenderInit(holdemrender_t *render) {
  render->batch = spriteBatchCreate(
    CARD_DECK_SIZE + (HOLDEM_PLAYER_COUNT * HOLDEM_PLAYER_HAND)
  );
  render->texture = assetTextureLoad("cards_normal.png");
  render->tileset = tilesetCreate(
    CARD_COUNT_PER_SUIT, 6,
    render->texture->width, render->texture->height,
    0, 0, 0, 0
  );
}

void holdemRenderCard(holdemrender_t *render, card_t card, bool faceUp, float x, float y, float z) {
  int32_t i;
  
  if(faceUp) {
    uint8_t suit = card / CARD_COUNT_PER_SUIT;
    uint8_t num = card % CARD_COUNT_PER_SUIT;
    i = num + ((
      suit == 0 ? 2 :
      suit == 1 ? 3 : 
      suit == 2 ? 1 :
      0
    ) * CARD_COUNT_PER_SUIT);
  } else {
    i = 4 * render->tileset->columns;
  }

  tilesetdiv_t *div = render->tileset->divisions + i;

  spriteBatchQuad(render->batch, -1,
    x, y, z,
    3, 4,
    div->x0, div->y0, div->x1, div->y1
  );
}

void holdemRender(holdemrender_t *render, holdemgame_t *game) {
  uint8_t i, j;
  holdemplayer_t *player;

  shaderUseTexture(GAME_STATE.shaderWorld, render->texture);

  // Flush
  spriteBatchFlush(render->batch);

  // Render Deck
  for(i = 0; i < game->deckSize; i++) {
    holdemRenderCard(render, game->deck[i], false, 0, 6, i * 0.05f);
  }
  for(i = 0; i < game->cardsFacing; i++) {
    holdemRenderCard(render, game->cards[i], true, 0, -i*1.5, i * 0.05f);
  }

  // Render each players's hand
  for(i = 0; i < HOLDEM_PLAYER_COUNT; i++) {
    player = game->players + i;

    for(j = 0; j < player->cardCount; j++) {
      holdemRenderCard(render, player->cards[j], true,
        4*(i+1), -(j * 1), j * 0.05f
      );
    }
  }

  // Draw
  spriteBatchDraw(render->batch, 0, -1);
}

uint32_t holdemBet(holdemgame_t *game, holdemplayer_t *player, uint32_t amount) {
  uint32_t realAmount = mathMin(player->chips, amount);
  game->pot += realAmount;
  player->chips -= realAmount;
  return realAmount;
}